#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <pcre.h>
#include "vars.h"

#define MAX 512
/************************************
 ********* Load PCRE file ***********/

void load_pcre_file(int section)
{
    FILE *fp;
    char tmp[MAX];
    const char *errstr; 
    unsigned int errchar; 


    //Set pointer
    redir[section]->ban_pcre=NULL;
    redir[section]->ban_pcre_s=NULL;
    redir[section]->ban_pcre_count=0;

    // Load pcre file
    sprintf(tmp,"%s/pcre",redir[section]->ban_dir);

    #ifdef DEBUG_PCRE
    printf("=== %s ===\n",tmp);
    #endif

    if ((fp=fopen(tmp,"r"))==NULL)
	{ //No file
          sprintf(tmp,"WARNING: Can't open file %s/pcre: %s",redir[section]->ban_dir,strerror(errno));
          err_mes(tmp);

	  #ifdef DEBUG_PCRE
          printf("%s\n",tmp);
	  #endif

        }
    else
    {
     while(!feof(fp))
      {
        //Get
        if (fgets(tmp,MAX,fp)!=NULL)
        {
    	    //Chomp
    	    if (tmp[strlen(tmp)-1]=='\n') tmp[strlen(tmp)-1]=0;
	    if (strlen(tmp)>0)
	      {

    		#ifdef DEBUG_PCRE
    		printf("%s\n",tmp);
    		#endif

        	//Add memory line
        	redir[section]->ban_pcre=(pcre **)realloc(redir[section]->ban_pcre,(1+redir[section]->ban_pcre_count)*sizeof(pcre *));
        	redir[section]->ban_pcre_s=(pcre_extra **)realloc(redir[section]->ban_pcre_s,(1+redir[section]->ban_pcre_count)*sizeof(pcre_extra *));
        	if (redir[section]->ban_pcre==NULL || redir[section]->ban_pcre_s==NULL)
                {
                 err_mes("Error: Can't allocate memory for pcre\n");
                 exit(-1);
                }

	       /******* Compile pattern *****************/    
	       if((redir[section]->ban_pcre[redir[section]->ban_pcre_count]=pcre_compile(tmp,PCRE_CASELESS,&errstr,&errchar,NULL))==NULL)
	       {
	        sprintf(tmp,"ERROR: Can't compile pattern in %s/pcre line:%i %s",redir[section]->ban_dir,redir[section]->ban_pcre_count+1,errstr);
	        err_mes(tmp);exit(-1);
	       }

	       /******* Optimize pattern ***************/
		redir[section]->ban_pcre_s[redir[section]->ban_pcre_count]=pcre_study(redir[section]->ban_pcre[redir[section]->ban_pcre_count],0,&errstr);    

		redir[section]->ban_pcre_count++;

	      }//if(strlen(tmp)>0)
       }//if fgets
     }//while

    fclose(fp);

    sprintf(tmp,"Load %i pattern from %s/pcre",redir[section]->ban_pcre_count,redir[section]->ban_dir);
    err_mes(tmp);
   }// if fopen
}

/******************************************
 ******** Check url by pcre rules *********/
 
 int check_pcre(int section)
 {
   unsigned int i;
   unsigned int vector[MAX];
   unsigned int vecsize=MAX;
   
    for (i=0;i<redir[section]->ban_pcre_count;i++)
    if(pcre_exec(redir[section]->ban_pcre[i],redir[section]->ban_pcre_s[i],input_url,strlen(input_url),0,PCRE_NOTEMPTY,vector,vecsize)>=0)
    {
            if(!redir[section]->log_flag)
	        {
		 fprintf(log,"%s %s %d.%d.%d.%d/%s %s (file:%s/pcre rule#:%d)\n",now(),redir[section]->name,input_ip[0],input_ip[1],input_ip[2],input_ip[3],input_ident,input_url,redir[section]->ban_dir,i+1);
//		 fprintf(log,"%s %s %d.%d.%d.%d/%s %s -> %s (file:%s/pcre rule#:%d)\n",now(),redir[z]->name,input_ip[0],input_ip[1],input_ip[2],input_ip[3],input_ident,input_url,redir[z]->url,redir[z]->ban_dir,flag);
		 fflush (log);
		}
     return(1);
    }
    return(0);
 }

